c------------------------------------------------------------------------------
c  Return a random magnitude error based on the combined
c  Gaussian plus log normal fit to the 2dF-SDSS magnitude
c  differences.
c 
c   The model for the distribution of magnitude differences,
c   d, is the sum of a Gaussian in d with sigma=0.14 containing
c   70% of the probability and Gaussian in ln(1+d) with sigma=0.235
c   containing the remaining 30% of the probability
c
c     
      real function  mag_error(iseed)
*************************************variables*********************************
      implicit none
      integer iseed,iseed1,iseed2,ifirst
      real lnd,frac
      real ran3,gasdev
      save iseed1,iseed2,ifirst
      data ifirst/0/
      logical magerr
      common /mag_err/ magerr
*******************************************************************************
      if (.not.magerr) then
         mag_error=0.
         return
      endif

c     Initialize random number seeds on first call
      if (ifirst.eq.0) then
         iseed1=iseed
         iseed2=iseed
         ifirst=1 
      end if

c     Choose randomly between the Gaussian and log Normal
c     components of the probability distribution
      frac=ran3(iseed1)
c
      if (frac.lt.0.7) then  ! 70% Gaussian
         mag_error=gasdev(iseed2)*0.14
      else                   ! 30% log normal
         lnd=gasdev(iseed2)*0.235
         mag_error=exp(lnd)-1.0
      end if
      return
      end
c------------------------------------------------------------------------------
c  Used to introduce a systematic error into mock and random
c  catalogues
c
      real function  mag_sys_error(zplus1)
*************************************variables*********************************
      implicit none
      real zplus1
      logical magerr
      common /mag_err/ magerr
*******************************************************************************
      if (.not.magerr) then
         mag_sys_error=0.
         return
      endif
c      mag_sys_error=   6.0*(zplus1-1.0775)**2 - 0.033
c      mag_sys_error=   0.03
      mag_sys_error=   0.0
      return
      end


c------------------------------------------------------------------------------
c  Used to introduce plate zero-point errors 
c  to catalogues
c
      real function  mag_plate_error(irandom,x,y,z,iseed)
*************************************variables*********************************
      implicit none
      real x,y,z,ra,dec,plate_errors(1000),PLATE_RMS,gasdev
c      parameter (PLATE_RMS=0.083) ! Estimate made for 100k magnitudes 
c                                  ! when compared to SDSS magnitudes
      parameter (PLATE_RMS=0.05) ! Rough estimate of rms value for 230k 
                                 ! magnitudes.  
      logical magerr
      integer ifirst,iplate,iseed,iseed1,irandom
      save ifirst,plate_errors
      data ifirst/0/
      common /mag_err/ magerr
*******************************************************************************
      if (.not.magerr) then
         mag_plate_error=0.
         return
      endif
c
c     On first call set up a table of offsets for each plate
      if (ifirst.eq.0) then
         ifirst=1
         iseed1=iseed
         do iplate=1,1000
            plate_errors(iplate)=gasdev(iseed1)*PLATE_RMS
         end do   
      end if

      if (irandom.eq.0) then
c        In the case of the randoms we want an independent
c        offset for each random galaxy
         mag_plate_error=gasdev(iseed1)*PLATE_RMS
      else
c        while for SOME mocks we MAY want the same offset across a 
c        whole plate        
c        Look up the zero-point error tabulated for this plate
         call xyz_radec(x,y,z,ra,dec)
         call fnumber(ra,dec,iplate)
         mag_plate_error=plate_errors(iplate)
      end if

      mag_plate_error=0.0
      return
      end

c------------------------------------------------------------------------------
c   Return a random redshift error based on a Gaussian fit to the 
c   redshift differences of multiply observed objects.
c
c   The model for the distribution of redshift differences, d,
c   is a Gaussian in d with sigma=85 km/s = 0.00001*85/3
c     
      real function  zz_error(iseed)
*************************************variables********************************
      implicit none
      integer iseed,iseed1,iseed2,ifirst
      real gasdev,sigma
      save iseed1,iseed2,ifirst,sigma
      data ifirst/0/
      logical zzerr
      common /zz_err/ zzerr
******************************************************************************
      if (.not.zzerr) then
         zz_error=0.
         return
      endif

c     Initialize random number seeds on first call
      if (ifirst.eq.0) then
         iseed1=iseed
         ifirst=1 
         sigma=85.0/3.0e+05
      end if

      zz_error=gasdev(iseed2)*sigma

      return
      end

c----------------------------------------------------------------------------
      FUNCTION gasdev(idum)
      INTEGER idum
      REAL gasdev
CU    USES ran3
      INTEGER iset
      REAL fac,gset,rsq,v1,v2,ran3
      SAVE iset,gset
      DATA iset/0/
      if (iset.eq.0) then
1       v1=2.*ran3(idum)-1.
        v2=2.*ran3(idum)-1.
        rsq=v1**2+v2**2
        if(rsq.ge.1..or.rsq.eq.0.)goto 1
        fac=sqrt(-2.*log(rsq)/rsq)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      END






